close all;
clear all;

%% Create the graph and define binary image function on it

n = 5500;
a = 1/3;
b = 0.45;
r = b*n^(-a);
rng('default');
[~, al, c, degs] = grgall_small_lists(n, r);

% Delaunay triangulation of the graph for presentation purposes
tri = delaunay(c(:, 1), c(:, 2));

% Shapes: two close disks
circle1 = (sqrt((c(:, 1)-0.37).^2 + (c(:, 2)-0.37).^2) < 0.15);
circle2 = (sqrt((c(:, 1)-0.63).^2 + (c(:, 2)-0.63).^2) < 0.15);
I = (circle1 | circle2) + 0;

figure;
plot(c(:, 1), c(:, 2), 'o', 'MarkerSize', 2);
hold on
plot(c(I > 0, 1), c(I > 0, 2), 'o',...
    'MarkerEdgeColor', 'k',...
    'MarkerFaceColor', 'r',...
    'MarkerSize', 3);
hold off
axis equal;
ylim([0, 1]);
xlim([0, 1]);

% Initial contour: circle containing the two disks
contour_internal = (sqrt((c(:, 1)-0.5).^2 + (c(:, 2)-0.5).^2) < 0.45);

%% Global parameters of the algorithm

% Gradient and curvature filters parameter
fil = [2, 2];

% Curvature approximation parameter
curv_approx = 0;

% Parameters of g function computation
sigma = 0.005;
lambda = 0.1;

% Total iterations parameter
iter = 120;

% Time evolution parameter
dt = 0.005;

% Balloon force regulation parameter
beta = -40;

%% Initialization of the algorithm on the graph

% Compute the fundamental geometric quantities of the graph essential for
% gradient and curvature calculations
[edgevectors, lengths, right_lengths, angles, omegas, index] =...
    edgevectors_angles(n, al, c, degs);

% Initialize the GAC algorithm
[g, g_grmag, g_grdir, Phi] = gac_init(I, c, sigma, lambda, contour_internal,...
    n, al, degs, edgevectors, lengths, angles, fil(1));

% Illustration of function g
figure;
trisurf(tri, c(:, 1), c(:, 2), g);
xlabel('x');
ylabel('y');
zlabel('g');
set(get(gca,'ZLabel'),'Rotation',0);

%% Main iteration stage of the algorithm

figure;
plot(c(:, 1), c(:, 2), 'o', 'MarkerSize', 2);
hold on
plot(c(Phi > 0, 1), c(Phi > 0, 2), 'o',...
    'MarkerEdgeColor', 'r',...
    'MarkerFaceColor', 'r',...
    'MarkerSize', 2);
hold off
axis equal;
ylim([0, 1]);
xlim([0, 1]);

for i=1:iter
    Phi = gac_iter(Phi, g, g_grmag, g_grdir, beta, dt, edgevectors, lengths,...
        angles, omegas, right_lengths, index, n, al, degs, fil, curv_approx);
    if ~mod(i, 40)
        figure;
        plot(c(:, 1), c(:, 2), 'o', 'MarkerSize', 2);
        hold on
        plot(c(Phi > 0, 1), c(Phi > 0, 2), 'o',...
            'MarkerEdgeColor', 'r',...
            'MarkerFaceColor', 'r',...
            'MarkerSize', 2);
        hold off
        axis equal;
        ylim([0, 1]);
        xlim([0, 1]);
    end
end

%% Comparison with initial shape

figure;
plot(c(:, 1), c(:, 2), 'o', 'MarkerSize', 2);
hold on
plot(c(Phi > 0, 1), c(Phi > 0, 2), 'o',...
    'MarkerEdgeColor', 'r',...
    'MarkerFaceColor', 'r',...
    'MarkerSize', 2);
plot(c(I > 0, 1), c(I > 0, 2), 'o',...
    'MarkerEdgeColor', 'g',...
    'MarkerFaceColor', 'g',...
    'MarkerSize', 2);
plot(c(I > 0 & Phi < 0, 1), c(I > 0 & Phi < 0, 2), 'o',...
    'MarkerEdgeColor', 'k',...
    'MarkerFaceColor', 'k',...
    'MarkerSize', 2);
hold off
axis equal;
ylim([0, 1]);
xlim([0, 1]);
