function c = curv_sum_pds(n, al, degs, grad, gradmagn,...
    edgevectors, lengths, angles, fil)

% Inputs:
%   n           - number of graph vertices
%   al          - adjacency list of the graph stored as an nxm matrix,
%               where m is maximal vertex degree on the graph
%   degs        - 1xn vector holding the degree of each vertex
%   grad        - nx2 matrix with approximate gradient field
%   gradmagn    - nx1 vector with magnitude of gradient field
%   edgevectors - mx2xn matrix (m = maximal vertex degree) each slice
%               of which contains the unit edge vectors starting from
%               the corresponding vertex in its first rows
%   lengths     - mx1xn matrix containing the edges' lengths
%               **modified** so that edges with zero length are
%               assigned a unit length, for proper division
%   angles      - mx1xn matrix holding in each slice the angles 
%               "occupied" by the corresponding vertex's neighbors
%   fil         - parameter controlling the type of filter used to smooth
%               the curvature approximation

% Outputs:
%   c           - nx1 vector holding the approximate curvature of the
%               embedding function's levelsets obtained with gradient-based
%               approximation

% Gradient magnitude modification to avoid ill 0/0 fractions in F field
% calculation
epsilon = 10^(-6);
gradmagn(gradmagn < epsilon) = epsilon;

% Unit gradient vector field (F) calculation
F = [grad(:,1)./gradmagn, grad(:,2)./gradmagn];

% Calculation of partial derivatives of F through gradient approximation
[grad_F1, ~] = gradient_approximation_step(n, al, degs, F(:,1),...
    edgevectors, lengths, angles);

[grad_F2, ~] = gradient_approximation_step(n, al, degs, F(:,2),...
    edgevectors, lengths, angles);

if (fil == 1)
    % Average filtering
    [grad_F1, ~] = gradient_average_filter(al, degs, n, grad_F1);
    [grad_F2, ~] = gradient_average_filter(al, degs, n, grad_F2);
    
else if (fil == 2)
        % Median filtering
        [grad_F1, ~] = gradient_median_filter(al, degs, n, grad_F1);
        [grad_F2, ~] = gradient_median_filter(al, degs, n, grad_F2);
    end
end

% Curvature computation as divergence (sum of respective partial
% derivatives) of vector field F
c = grad_F1(:,1) + grad_F2(:,2);

end

