function [g, g_grmag, g_grdir, phi_0] = gac_init(I, c, sigma, lambda, contour_internal,...
    n, al, degs, edgevectors, lengths, angles, fil)

% Inputs:
%   I                   - initial intensity function on the graph vertices
%   c                   - nx2 matrix holding the positions of the vertices
%   sigma               - scale of the graph-theoretic Gaussian smoothing
%                       filter
%   lambda              - scale parameter in formula for computation of g
%                       function
%   contour_internal    - nx1 logical vector determining which of the
%                       graph's vertices belong to the given initial
%                       contour's internal
%   n                   - number of graph vertices
%   al                  - adjacency list of the graph stored as an nxm
%                       matrix, where m is maximal vertex degree on the
%                       graph
%   degs                - 1xn vector holding the degree of each vertex
%   edgevectors         - mx2xn matrix (m = maximal vertex degree) each
%                       slice of which contains the unit edge vectors
%                       starting from the corresponding vertex in its first
%                       rows
%   lengths             - mx1xn matrix containing the edges' lengths
%                       **modified** so that edges with zero length are
%                       assigned a unit length, for proper division
%   angles              - mx1xn matrix holding in each slice the angles 
%                       "occupied" by the corresponding vertex's neighbors
%   fil                 - parameter controlling the type of filter used to
%                       smooth the g function's gradient approximation

% Outputs:
%   g       - nx1 vector holding the values of g on the graph's vertices         
%   g_grmag - nx1 vector with the approximate gradient magnitude of g
%           function
%   g_grdir - nx2 matrix holding the approximate gradient field of g
%           function
%   phi_0   - nx1 vector with approximate signed distance transform from
%           the initial contour

% Initializes the Geodesic Active Contours Algorithm by computing the
% initial embedding function phi_0, the edges' strength function g, its
% gradient magnitude g_grmag and its direction g_grdir

%% Signed distance function computation

phi_0 = dist_given_contour(contour_internal, c, al, n, degs);

%% g function computation

[g, I_s_grmag] = g_function_maxapprox_grad(I, c, n, al, degs, sigma, lambda);
% [g, I_s_grmag] = g_function_gauss_grad_sep_nrmlz(I, c, n, al, degs, sigma, lambda);

%% g function's gradient direction calculation using new approximation

[g_grad, g_gradmagn] = gradient_approximation_step(n, al, degs, g,...
    edgevectors, lengths, angles);
if (fil == 1)
    % Average filtering
    [g_grad, g_gradmagn] = gradient_average_filter(al, degs, n, g_grad);
        
else if (fil == 2)
        % Median filtering
        [g_grad, g_gradmagn] = gradient_median_filter(al, degs, n, g_grad);
    end
end

% Gradient magnitude modification to avoid ill 0/0 fractions in F field
% calculation
epsilon = 10^(-6);
g_gradmagn(g_gradmagn < epsilon) = epsilon;

g_grdir = [g_grad(:,1)./g_gradmagn, g_grad(:,2)./g_gradmagn];

%% g function's gradient magnitude computation with new or old faithful approximation

%g_grmag = g_gradmagn;
g_grmag = gradmagn_max(n, al, degs, g);

end