function [Phi_next] = gac_iter(Phi, g, g_grmag, g_grdir, beta, dt, edgevectors, lengths, angles,...
    omegas, right_lengths, index, n, al, degs, fil, curv_approx)

% Inputs:
%   Phi             - the embedding function in the previous step
%   g               - nx1 vector holding the values of g on the graph's         
%                   vertices
%   g_grmag         - nx1 vector with the approximate gradient magnitude of
%                   g function
%   g_grdir         - nx2 matrix holding the approximate gradient field of
%                   g function
%   beta            - constant factor regulating the relative force of the
%                   balloon term w.r.t. the curvature and spring terms
%   dt              - time step determining to which extent the embedding
%                   function is updated
%   edgevectors     - mx2xn matrix (m = maximal vertex degree) each slice
%                   of which contains the unit edge vectors starting from
%                   the corresponding vertex in its first rows                     
%   lengths         - mx1xn matrix containing the edges' lengths
%                   **modified** so that edges with zero length are
%                   assigned a unit length, for proper division
%   angles          - mx1xn matrix holding in each slice the angles 
%                   "occupied" by the corresponding vertex's neighbors
%   omegas          - mx1xn matrix holding in each slice the angles 
%                   assigned to each neighbor of the corresponding vertex
%   right_lengths   - mx1xn matrix containing the actual edges' lengths
%   index           - mx1xn matrix holding in each slice the indices of the
%                   adjacency list of the corresponding vertex in an order
%                   such that the neighbors are sorted in ascending order
%                   w.r.t. the edge's angle with the x-axis
%   n               - number of graph vertices
%   al              - adjacency list of the graph stored as an nxm matrix,
%                   where m is maximal vertex degree on the graph          
%   degs            - 1xn vector holding the degree of each vertex
%   fil             - 1x2 vector controlling the type of filter (average or
%                   median) used to smooth the gradient and curvature
%                   approximations
%   curv_approx     - parameter determining the kind of approximation used
%                   to compute the curvature; differential or integral

% Outputs:
%   Phi_next        - the embedding function after the current step

% Gradient computation

[gr_Phi, grmag_Phi] = gradient_approximation_step(n, al, degs, Phi,...
    edgevectors, lengths, angles);

if (fil(1) == 1)
    % Average filtering
    [gr_Phi, grmag_Phi] = gradient_average_filter(al, degs, n, gr_Phi);
        
else if (fil(1) == 2)
        % Median filtering
        [gr_Phi, grmag_Phi] = gradient_median_filter(al, degs, n, gr_Phi);
    end
end

% Curvature computation

if curv_approx
    % Method 2 for curvature - Geometric approximation
    curv_Phi = curv_geom(al, degs, n, angles, omegas, right_lengths, index,...
        gr_Phi, grmag_Phi, fil(2));
else
    % Method 1 for curvature - Gradient-based approximation
    curv_Phi = curv_sum_pds(n, al, degs, gr_Phi, grmag_Phi,...
        edgevectors, lengths, angles, fil(2));
end

% Maximum filtering on curvature to avoid very low values
curv_Phi = max_filter(al, degs, n, curv_Phi);

% Unit gradient vector field (F) calculation using new approximation

% Gradient magnitude modification to avoid ill 0/0 fractions in F field
% calculation
epsilon = 10^(-6);
grmag_Phi(grmag_Phi < epsilon) = epsilon;

F = [gr_Phi(:,1)./grmag_Phi, gr_Phi(:,2)./grmag_Phi];

% Gradient magnitude computation with old faithful approximation

grmag_Phi_max = gradmagn_max(n, al, degs, Phi);

% GAC partial difference equation

% Update embedding function Phi to produce output Phi_next according to the
% partial difference equation constituting the G.A.C. algorithm

Phi_next = Phi + dt*(g.*(curv_Phi + beta).*grmag_Phi_max...
    + sum(g_grdir.*F, 2).*g_grmag.*grmag_Phi_max);

% Median filtering

% Filter the updated embedding function with median filter to smooth the
% result
Phi_next = median_filter(al, degs, n, Phi_next);
% Phi_next = average_filter(al, degs, n, Phi_next);

end

