function [grad, gradmagn] = gradient_approximation_step(n, al, degs, u,...
    edgevectors, lengths, angles)

% Inputs:
%   n           - number of graph vertices
%   al          - adjacency list of the graph stored as an nxm matrix,
%               where m is maximal vertex degree on the graph
%   degs        - 1xn vector holding the degree of each vertex
%   u           - real function on graph vertices as an nx1 vector
%   edgevectors - mx2xn matrix (m = maximal vertex degree) each slice
%               of which contains the unit edge vectors starting from
%               the corresponding vertex in its first rows
%   lengths     - mx1xn matrix containing the edges' lengths
%               **modified** so that edges with zero length are
%               assigned a unit length, for proper division
%   angles      - mx1xn matrix holding in each slice the angles 
%               "occupied" by the corresponding vertex's neighbors

% Outputs:
%   grad        - nx2 matrix with approximate gradient field
%   gradmagn    - nx1 vector with magnitude of gradient field

m = max(degs);

% Defining the differences in the values of the image function, u(w) - u(v),
% between neighboring vertices

differences = zeros(m, 1, n);
for i=1:n
    differences(1:degs(i), 1, i) = u(al(i, 1:degs(i))) - u(i);
end

% Calculation of gradient approximation 

grad = zeros(n, 2);
grad(:, 1) = sum(((differences.*edgevectors(:, 1, :)).*angles)./lengths)/pi;
grad(:, 2) = sum(((differences.*edgevectors(:, 2, :)).*angles)./lengths)/pi;
gradmagn = sqrt(grad(:,1).^2 + grad(:, 2).^2);

end

